#! /usr/bin/perl
##############################################################################
# Gadget Renesas Web Compiler's project files downloader
#
# Copyright 2012-2013 Yuuichi Akagawa
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
#
# 2012.09.20 New
# 2013.05.22 Skip gr_writer directory
#
##############################################################################
#
# Usage: gr_backup.pl ProjectName
#
##############################################################################
use strict;
use warnings;

use LWP::Simple;
use LWP::UserAgent;
use JSON qw/decode_json/;
#use Data::Dumper;
#$|=1;

#set your API KEY
my $API_KEY='INPUT_YOUR_API_KEY';

my $uri = 'http://tool-cloud.renesas.com/system/api/';

#check argument
die "usage: gr_backup.pl PROJECT_NAME\n" if($#ARGV != 0);

#Project Name
my $pjname = shift(@ARGV);

my $ua = new LWP::UserAgent;
$ua->agent("AgentName/0.1 " . $ua->agent);

#Get Project ID
my $pjid = &getpjid($pjname) or die "Error: Project not found\n"; 
mkdir($pjname) or die "Error: mkdir $pjname failed.\n"; 
chdir($pjname) or die "Error: chdir $pjname failed.\n"; 

#Do backup
&filelist(0);

chdir('..');


sub filelist{
    my $pid = shift(@_);

    my $req = new HTTP::Request GET => $uri . '?module=structure&action=list&key=' . $API_KEY . '&project=' . $pjid . '&parent=' . $pid;
    my $reqres = $ua->request($req);

    if($reqres->is_success){
#print Dumper($reqres->content), "\n";
        if( $reqres->content eq '' ){
            return -1;
        }
        my $data = decode_json($reqres->content);

        foreach my $pj (@{$$data{structure}}){
            my $objid   = $$pj{id};
            my $objname = $$pj{name};
            my $objtype = $$pj{type};

            #file
            if($objtype == 2){
                $req = new HTTP::Request GET => $uri . '?module=object&action=getdata&key=' . $API_KEY . '&id=' . $objid;
                $reqres = $ua->request($req);
                if( $reqres->is_success){
                    open(FH, ">$objname");
                    binmode FH;
                    print FH $reqres->content;
                    close(FH);
                }else{
                    print "Error: File get failed.($objname)\n";
                }
            }

            #directory
            if($objtype == 1){
                next if($objname eq 'gr_writer'); #Skip KurumiWriter
                mkdir($objname);
                chdir($objname);
                if(&filelist($objid) == -1){
                    print "Error: directory info get failed. '$objname' skipped.\n";
                }
                chdir('..');
            }
        }
    }
    return 0;
}

sub getpjid {
    my $pjname = shift(@_);
    my $req = new HTTP::Request GET => $uri . '/?module=projects&action=list&key=' . $API_KEY;
    my $reqres = $ua->request($req);
    if( $reqres->is_success){
        my $data = decode_json($reqres->content);

        foreach my $pj (@{$$data{projects}}){
	    if($$pj{name} eq $pjname){
		return $$pj{id};
	    }
        }
    }
    return(undef);
}


